import React, { useState, useEffect } from 'react';
import ColorPicker from '@graphpolaris/shared/lib/components/colorComponents/colorPicker';
import { Button, EntityPill, Icon, Input, RelationPill } from '@graphpolaris/shared/lib/components';
import { MapLayerSettingsPropTypes } from '..';

export function NodeLinkOptions({ settings, graphMetadata, updateSettings, spatialAttributes }: MapLayerSettingsPropTypes) {
  const handleCollapseToggle = (nodeType: string) => {
    settings.nodes[nodeType].collapsed = !settings.nodes[nodeType].collapsed;
    updateSettings({ nodes: settings.nodes });
  };

  useEffect(() => {
    graphMetadata.nodes.labels.map((nodeType) => {
      if (settings?.[nodeType]?.lat) {
      }
    });
  }, [settings.node, graphMetadata]);

  return (
    <div>
      {settings?.nodes &&
        Object.keys(settings.nodes).map((nodeType) => {
          const nodeSettings = settings.nodes[nodeType];
          return (
            <div className="mt-2" key={nodeType}>
              <div className="flex items-center">
                <div className="flex flex-grow mr-2 cursor-pointer" onClick={() => handleCollapseToggle(nodeType)}>
                  <EntityPill title={nodeType} />
                </div>
                <div className="flex items-center space-x-2">
                  <ColorPicker
                    value={nodeSettings.color}
                    updateValue={(val) => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, color: val } } });
                    }}
                  />
                  <Button
                    iconComponent={nodeSettings.hidden ? 'icon-[ic--baseline-visibility-off]' : 'icon-[ic--baseline-visibility]'}
                    variant="ghost"
                    onClick={() => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, hidden: !nodeSettings.hidden } } });
                    }}
                  />
                </div>
              </div>

              {!nodeSettings.collapsed && (
                <div>
                  <Input
                    inline
                    label="Latitude"
                    type="dropdown"
                    value={nodeSettings.lat}
                    options={[...(spatialAttributes[nodeType] || [])]}
                    disabled={!settings.node || spatialAttributes[nodeType].length < 1}
                    onChange={(val) => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, lat: String(val) } } });
                    }}
                  />
                  <Input
                    inline
                    label="Longitude"
                    type="dropdown"
                    value={nodeSettings.lon}
                    options={[...(spatialAttributes[nodeType] || [])]}
                    disabled={!settings.node || spatialAttributes[nodeType].length < 1}
                    onChange={(val) => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, lon: String(val) } } });
                    }}
                  />
                  <Input
                    inline
                    label="Shape"
                    type="dropdown"
                    value={nodeSettings.shape}
                    options={['circle', 'square', 'triangle', 'diamond', 'location', 'star']}
                    disabled={!settings.shape}
                    onChange={(val) => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, shape: String(val) as any } } });
                    }}
                  />
                  <Input
                    label="Size"
                    type="slider"
                    min={0}
                    max={40}
                    step={1}
                    value={nodeSettings.size}
                    onChange={(val) => {
                      updateSettings({ nodes: { ...settings.nodes, [nodeType]: { ...nodeSettings, size: Number(val) } } });
                    }}
                  />
                </div>
              )}
            </div>
          );
        })}

      {settings?.edges &&
        Object.keys(settings.edges).map((edgeType) => {
          const edgeSettings = settings.edges[edgeType];

          return (
            <div className="mt-2" key={edgeType}>
              <div className="flex items-center">
                <div className="w-3/4 mr-6 cursor-pointer" onClick={() => handleCollapseToggle(edgeType)}>
                  <RelationPill title={edgeType} />
                </div>
                <div className="w-1/2 flex">
                  <ColorPicker
                    value={settings.edges[edgeType].color}
                    updateValue={(val) => updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, color: val } } })}
                  />
                  <Button
                    iconComponent={
                      settings.edges[edgeType].hidden ? 'icon-[ic--baseline-visibility-off]' : 'icon-[ic--baseline-visibility]'
                    }
                    variant="ghost"
                    onClick={() =>
                      updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, hidden: !edgeSettings.hidden } } })
                    }
                  />
                </div>
              </div>

              {!edgeSettings.collapsed && (
                <div>
                  <Input
                    label="Enable brushing"
                    type="boolean"
                    value={settings.enableBrushing}
                    onChange={(val) => {
                      updateSettings({ enableBrushing: val as boolean });
                    }}
                  />

                  <div>
                    <div className="flex items-center gap-1">
                      <Icon component="icon-[ic--baseline-subdirectory-arrow-right]" size={16} color="text-secondary-300" />
                      <span>Width</span>
                    </div>
                    <Input
                      label="Fixed"
                      type="boolean"
                      value={edgeSettings.fixed}
                      onChange={(val) => updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, fixed: val } } })}
                    />
                    {!edgeSettings.fixed ? (
                      <div>
                        <Input
                          label="Based on"
                          type="dropdown"
                          size="xs"
                          options={
                            graphMetadata.nodes.types[edgeType]?.attributes
                              ? Object.keys(graphMetadata.nodes.types[edgeType].attributes).filter(
                                  (key) => graphMetadata.nodes.types[edgeType].attributes[key].dimension === 'numerical',
                                )
                              : []
                          }
                          value={edgeSettings.sizeAttribute ?? ''}
                          onChange={(val) =>
                            updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, sizeAttribute: String(val) } } })
                          }
                        />
                        <div className="flex">
                          <Input
                            type="number"
                            label="min"
                            size="xs"
                            value={edgeSettings.min}
                            onChange={(val) => updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, min: val } } })}
                          />
                          <Input
                            type="number"
                            label="max"
                            size="xs"
                            value={edgeSettings.max}
                            onChange={(val) => updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, max: val } } })}
                          />
                        </div>
                      </div>
                    ) : (
                      <div>
                        <Input
                          type="slider"
                          label="Width"
                          min={0}
                          max={10}
                          step={0.2}
                          value={settings.edges[edgeType].width}
                          onChange={(val) =>
                            updateSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, width: Number(val) } } })
                          }
                        />
                      </div>
                    )}
                  </div>
                </div>
              )}
            </div>
          );
        })}
    </div>
  );
}
