import { CompositeLayer, Position } from 'deck.gl';
import { MapLayerSettingsPropTypes } from './layers';
import { VisualizationPropTypes, VisualizationSettingsType } from '../../common';
import { MapProps } from './mapvis';
import { Node as QueryNode } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';

export type Coordinate = [number, number] | [];

export interface LayerProps {
  [key: string]: any;
}

export type CompositeLayerType = VisualizationPropTypes<MapProps> & {
  selected: any[];
  hoverObject: QueryNode | null;
  getNodeLocation: (d: string) => Position;
  flyToBoundingBox: (minLat: number, maxLat: number, minLon: number, maxLon: number, options?: { [key: string]: any }) => void;
  setLayerIds: (val: string[]) => void;
};

export type Layer = {
  id: string;
  component: CompositeLayer<CompositeLayerType>;
  settings: MapProps & VisualizationSettingsType;
};

export type Node = {
  id: string;
  attributes: {
    long: number;
    lat: number;
    [key: string]: any;
  };
  connectedEdges: string[];
  [key: string]: any;
};

export type Edge = {
  id: string;
  from: string;
  to: string;
  attributes: {
    [key: string]: any;
  };
  [key: string]: any;
};
